#!/usr/bin/perl
# ---------------------------------------------------------------
# phone.cgi
# ---------
# This application enables an administrator to add, modify and
# remove configuration information from the asterisk system.
# ---------------------------------------------------------------
# COPYRIGHT_INFORMATION
# ---------------------------------------------------------------
# Revision History:
# -----------------
# 04/17/2001 - Mike Patton <mpatton@linux-support.net>
#              Initial Development Version
# ---------------------------------------------------------------

require '../web-lib.pl';
ReadParse();

startPage();
if ( $in{action} eq 'Add Extension' ) {
   $in{extension} = '';
   showForm();
} elsif ( $in{action} eq 'Edit Extension' ) {
   if ( $in{extension} ) {
      loadItem();
   }
   showForm();
} elsif ( $in{action} eq 'save' ) {
   saveItem();
} elsif ( $in{action} eq 'Delete Extension' ) {
   deleteItem();
} else {
   listItems();
}

endPage();

sub startPage {
   print qq[Content-type: text/html

<html>
  <body bgcolor="#FFFF00">];
}

sub endPage {
   print "</body></html>\n";
}

sub deleteItem {
   if ( $in{confirmed} ) {

      open( CONFIG, 'data/extensions.dat' );
      my @configData = <CONFIG>;
      close( CONFIG );

      open( CONFIG, '>data/extensions.dat' );
      foreach ( @configData ) {
         my ( $ext, $name, $email, $voicemail ) = split( /\|/, $_ );
         if ( $ext && $ext ne $in{extension} ) {
            print CONFIG "$_";
         }
      }
      close(CONFIG);

      open( CONFIG, '/etc/asterisk/extensions.conf' );
      my @confData = <CONFIG>;
      close( CONFIG );
      remVoicemailDir();
      open( CONFIG, '>/etc/asterisk/extensions.conf.new' );
      foreach ( @confData ) {
         if ( $_ =~ /^exten=(\d+)/ ) {
            next if $1 eq $in{extension};
         }
         print CONFIG $_;
      }
      close(CONFIG);

      open( CONFIG , '/etc/asterisk/voicemail.conf');
      my @config = <CONFIG>;
      close(CONFIG);
      open( CONFIG, '>/etc/asterisk/voicemail.conf.new');
      foreach ( @config ) {
         next if $_ =~ /^$in{extension}/;
         print CONFIG $_;
      }
      close(CONFIG);

      listItems();
   } else {
      my ( $ext, $name, $email, $voicemail ) = getExtensionData($in{extension});
      print qq[
<center>
  <font face="arial" size="2">You are about to remove <b>$name ( $in{extension} )</b>. Are you sure?
  <br><br>
  <form>
  <input type="hidden" name="extension" value="$in{extension}">
  <input type="hidden" name="action" value="Delete Extension">
  <input type="hidden" name="confirmed" value="1">
  <input type="submit" value="Yes!">
  </form>
</center>
      ];
   }
}

sub loadItem {
   ( $in{extension}, $in{password}, $in{name}, $in{email}, $in{voicemail}, $in{interface} ) = getExtensionData( $in{extension} );
}

sub saveItem {
   my ( $err );
   my $extensionEntry = createExtensionEntry();

   if ( $in{interface} =~ /^Other/ ) {
      $in{interface} = "Other^$OtherValue";
   }

   my $voicemailEntry = createVoicemailEntry() if $in{voicemail};

   if ( ! $in{extension} ) {
      $err .= "<li>The extension is a required field, but it was left blank.</li>\n";
   }

   if ( ! $in{name} ) {
      $err .= "<li>The name is a required field, but it was left blank.</li>\n";
   }

   if ( ! $in{password} and $in{voicemail} eq 'on' ) {
      $err .= "<li>You must specify a password if the user is to have voicemail.</li>\n";
   }

   if ( $in{subAction} eq 'Add Extension' ) {

      my ( $tmpExt, @junk ) = checkExtensionData($in{extension});

      if ( $tmpExt ) {
         $err .= "<li>The extension $in{extension} is already in use. Please try again.</li>\n";
      }

      if ( $in{extension} !~ /^\d+$/ ) {
         $err .= "<li>The extension can only contain numbers.</li>";
      }

      if ( $in{password} and $in{password} !~ /^\d+$/ ) {
         $err .= "<li>The password can only contain numbers.</li>";
      }

      if ( ! $err ) {

         open( CONFIG, '>>data/extensions.dat' );
         print CONFIG "$in{extension}|$in{password}|$in{name}|$in{email}|$in{voicemail}|$in{interface}\n";
         close( CONFIG );

         open( CONFIG, '/etc/asterisk/extensions.conf' );
         my @confData = <CONFIG>;
         close( CONFIG );

         open( CONFIG, '>/etc/asterisk/extensions.conf' );
         my $seen = 0;
         foreach ( @confData ) {
            print CONFIG $_;
            if ( $_ =~ /^\[default\]/g and not $seen ) {
               $seen = 1;
               print CONFIG "$extensionEntry\n";
            }
         }
         close(CONFIG);
         createVoicemail();

         open( CONFIG , '/etc/asterisk/voicemail.conf');
         my @config = <CONFIG>;
         close(CONFIG);
         open( CONFIG, '>/etc/asterisk/voicemail.conf');
         foreach ( @config ) {
            next if $_ =~ /^$in{extension}/;
            print CONFIG $_;
            if ( $_ =~ /^\[default\]/ ) {
               if ( $in{voicemail} ) {
                  print CONFIG "$voicemailEntry\n";
               }
            }
         }
         close(CONFIG);
      }

   } elsif ( $in{subAction} eq 'Edit Extension' ) {

      my ( $tmpExt, @junk ) = getExtensionData($in{extension});

      if ( ! $tmpExt ) {
         $err .= "<li>The extension $in{extension} does not exist. Please try again.</li>\n";
      }

      if ( ! $err ) {
         open( CONFIG, 'data/extensions.dat' );
         my @configData = <CONFIG>;
         close( CONFIG );

         open( CONFIG, '>data/extensions.dat' );
         foreach ( @configData ) {
            my ( $ext, $password, $name, $email, $voicemail ) = split( /\|/, $_ );
            if ( $ext && $ext ne $in{extension} ) {
               print CONFIG "$_";
            } else {
               print CONFIG "$ext|$in{password}|$in{name}|$in{email}|$in{voicemail}|$in{interface}\n";
            }
         }
         close(CONFIG);

         open( CONFIG, '/etc/asterisk/extensions.conf' );
         my @confData = <CONFIG>;
         close( CONFIG );

         open( CONFIG, '>/etc/asterisk/extensions.conf' );
         foreach ( @confData ) {
            if ( $_ =~ /^exten=(\d+)/ ) {
               next if $1 eq $in{extension};
            }
            print CONFIG $_;
            if ( $_ =~ /^\[default\]/g ) {
               print CONFIG "$extensionEntry\n";
            }
         }
         close(CONFIG);

         open( CONFIG , '/etc/asterisk/voicemail.conf');
         my @config = <CONFIG>;
         close(CONFIG);
         open( CONFIG, '>/etc/asterisk/voicemail.conf');
         foreach ( @config ) {
            next if $_ =~ /^$in{extension}/;
            print CONFIG $_;
            if ( $_ =~ /^\[default\]/ ) {
               if ( $in{voicemail} ) {
                  print CONFIG "$voicemailEntry\n";
               }
            }
         }
         close(CONFIG);

      }

   }

   if ( $err ) {
      showForm( $err );
      return;
   } else {
      system("/usr/sbin/asterisk -r -x reload");
      listItems();
   }

}

sub listItems {
   open( CONFIG , 'data/extensions.dat' );
   my @extensions = <CONFIG>;
   close( CONFIG );
   print qq[
<form>
  <center>
    <table border="0" cellpadding="0" cellspacing="0">
      <tr>
        <td bgcolor="#000000">
          <table border="0" cellpadding="3" cellspacing="1">
            <tr valign="top">
              <td bgcolor="#FFFFFF">
                <font face="arial" size="2">
                  <select name="extension">];
   my %extensions;
   foreach ( @extensions ) {
      my ( $ext, $pass, $name, @junk ) = split(/\|/, $_);
      $extensions{$ext} = $name;
   }

   foreach my $ext ( sort keys %extensions ) {
      print "<option value=$ext>$extensions{$ext} ( $ext )";
   }

   print qq[
                  </select>
                </font>
              </td>
              <td bgcolor="#FFFFFF">
                <font face="arial" size="2">
                  <input type="submit" name="action" value="Edit Extension">
                  <input type="submit" name="action" value="Delete Extension">
                  <input type="submit" name="action" value="Add Extension">
                </font>
              </td>
            </tr>
          </table>
        </td>
      </tr>
    </table>
  </center>
</form>];

}

sub checkExtensionData {
   my $target = shift;
   my ( $ext, @data ) = getExtensionData($target);
   if ( $ext ) {
      return($ext, @data);
   }
   open(DATA, "/etc/asterisk/extensions.conf");
   while(<DATA>) {
      $_ =~ s/^\s+|\s+$//g;
      next if $_ =~ /^\;/;
      next if not $_;
      if ( $_ =~ /^exten=(.+)/ ) {
         my ( $ext, @data ) = split(/\,/, $1);
         if ( $ext eq $target ) {
            return( $ext, @data );
         }
      }
   }
   return 0;
}
      

sub getExtensionData {

   my $target = shift;

   open( CONFIG, 'data/extensions.dat' );
   my @configData = <CONFIG>;
   close( CONFIG );

   foreach ( @configData ) {
      chomp();
      my ( $ext, $password, $name, $email, $voicemail, $interface ) = split( /\|/ , $_ );
      if ( $ext eq $target ) {
         return( $ext, $password, $name, $email, $voicemail, $interface );
      }
   }

}

sub showForm {
   my $error = shift;
   if ( $in{action} eq 'save' ) {
      $in{action} = $in{subAction};
   }
   if ( $error ) {
      print "<br><br>There were errors with the previous form. They were:<br><ul>$error</ul>Please make the appropriate changes below and re-submit the form.";
   }
   if ( $in{voicemail} eq 'on' ) {
      $checked = ' CHECKED';
   }

   if ( ! $in{interface} ) {
      $in{interface} = "Console/dsp";
   }
 
   $otherValue = '';
   if ( $in{interface} =~ /^Other\^(.+)$/ ) {
      $otherValue = $1;
   }

   print qq[
<center>
  <form>
    <input type="hidden" name="action" value="save">
    <input type="hidden" name="subAction" value="$in{action}">
    <table border="0" cellpadding="0" cellspacing="0">
      <tr>
        <td bgcolor="#000000">
          <table border="0" cellpadding="2" cellspacing="1" width="100%">
            <tr>
              <td bgcolor="#FFFFFF"><font face="arial" size="2">Ext.:<br>];
   if ( $in{action} eq 'Add Extension' ) {
      print qq[<input type="text" name="extension" value="$in{extension}" size="4">];
   } else {
      print qq[<input type="hidden" name="extension" value="$in{extension}">$in{extension}];
   }
   print qq[</font></td>
             <td bgcolor="#FFFFFF"><font face="arial" size="2">Interface:<br><select name="interface">]; &getInterfaces($in{interface}); print qq[</select>&nbsp;&nbsp;<input type="text" name="OtherValue" size="10" value="$OtherValue"></font></td>
              <td bgcolor="#FFFFFF"><font face="arial" size="2">Name:<br><input type="text" name="name" value="$in{name}" size="20"></font></td>
</tr><tr>
              <td bgcolor="#FFFFFF"><font face="arial" size="2">Password:<br><input type="text" name="password" value="$in{password}" size="4"></font></td>
              <td bgcolor="#FFFFFF"><font face="arial" size="2">E-Mail:<br><input type="text" name="email" value="$in{email}" size="20"></font></td>
              <td bgcolor="#FFFFFF"><font face="arial" size="2"><input type="checkbox" name="voicemail"$checked>Voicemail</font></td>
            </tr>
          </table>
        </td>
      </tr>
      <tr>
        <td>
          <center>
            <table border="0" cellpadding="5" cellspacing="0">
              <tr>
                <td><font face="arial" size="2"><input type="submit" value="Save"></font></td>
                <td><font face="arial" size="2"><input type="reset" value="Reset"></font></td>
              </tr>
            </table>
          </center>
        </td>
      </tr>
    </table>
  </form>
</center>];
   return 0;
}

sub selectInterface {
   my $interface = shift;
}

sub createExtensionEntry {
   if ( $in{interface} =~ /^Other\^(.+)$/ ) {
      $in{interface} = $1;
   }
   if ( $in{voicemail} ) {
      return qq[exten=$in{extension},1,Playback,transfer ; "Please hold while..."
exten=$in{extension},2,Dial,$in{interface}|10        ; Ring the console, 10 secs max
exten=$in{extension},3,Playback,vm/$in{extension}/unavail   ; "I'm not here right now"
exten=$in{extension},4,Voicemail,$in{extension}             ; Send to voicemail...
exten=$in{extension},5,Goto,s|6                   ; Start over
exten=$in{extension},103,Playback,vm/$in{extension}/busy    ; (2 + 101) "I'm on the phone"
exten=$in{extension},104,Goto,4                   ; Go to voicemail, etc.];
   } else {
      return qq[exten=$in{extension},1,Dial,$in{interface}           ; Ring forever];
   }
}

sub createVoicemail {
   system("mkdir /var/spool/asterisk/vm/$in{extension}");
   system("mkdir /var/spool/asterisk/vm/$in{extension}/INBOX");
   my @extArr = split(//, $in{extension});
   my $extension;
   foreach ( @extArr ) {
      $extension .= " /var/lib/asterisk/sounds/digits/$_.gsm";
   }
   system("cat /var/lib/asterisk/sounds/vm-theperson.gsm $extension /var/lib/asterisk/sounds/vm-isonphone.gsm > /var/spool/asterisk/vm/$in{extension}/busy.gsm");
   system("cat /var/lib/asterisk/sounds/vm-theperson.gsm $extension /var/lib/asterisk/sounds/vm-isunavail.gsm > /var/spool/asterisk/vm/$in{extension}/unavail.gsm");
}

sub remVoicemailDir {
   system("rm -rf /var/spool/asterisk/vm/$in{extension}");
}

sub createVoicemailEntry {
   return qq[$in{extension}=$in{password},$in{name},$in{email}];
}

sub getInterfaces {
   my $int = shift;
   print "<option value=\"Other\">Other->";
   my $selected = '';
   if ( $int eq "Console/dsp" ) {
      $selected = ' SELECTED';
   }
   print "<option value=\"Console/dsp\"$selected>Console/dsp";
   open(CONF, "/etc/tormenta.conf");
   my @channels;
   while(<CONF>) {
      $_ =~ s/\;.+$//;
      $_ =~ s/^\s+|\s+$//;
      next if $_ !~ /^fxols/ and $_ !~ /^fxoks/ and $_ !~ /^fxogs/;
      my ( $type, $range ) = split(/=/, $_);
      my @ranges = split(/\,/, $range);
      foreach my $range ( @ranges ) {
         my @varRange = split(/-/, $range);
         if ( $varRange[0] && $varRange[1] ) {
            for ( my $x = $varRange[0] ; $x <= $varRange[1] ; $x++ ) {
               my $selected = '';
               if ( $int eq "Tor/$x" ) {
                  $selected = ' SELECTED';
               }
               print "<option value=\"Tor/$x\"$selected>Tor/$x";
            }
         } else {
            my $selected = '';
            if ( $int eq "Tor/$varRange[0]" ) {
               $selected = ' SELECTED';
            }
            print "<option value=\"Tor/$varRange[0]\"$selected>Tor/$varRange[0]";
         }
      }
   }
   
}
